@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000-2001,2005 Silicon Graphics, Inc.
+ * Copyright (c) 2000-2001,2005-2006 Silicon Graphics, Inc.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -113,9 +113,9 @@ libxfs_trans_iget(
 	xfs_inode_log_item_t	*iip;
 
 	if (tp == NULL)
-		return libxfs_iread(mp, tp, ino, ipp, 0);
+		return libxfs_iget(mp, tp, ino, lock_flags, ipp, 0);
 
-	error = libxfs_iread(mp, tp, ino, &ip, 0);
+	error = libxfs_iget(mp, tp, ino, lock_flags, &ip, 0);
 	if (error)
 		return error;
 	ASSERT(ip != NULL);
@@ -310,14 +310,14 @@ libxfs_trans_brelse(
 		bip->bli_recur--;
 		return;
 	}
-	/* If dirty, can't release till transaction committed */
-	if (lidp->lid_flags & XFS_LID_DIRTY) {
+	/* If dirty/stale, can't release till transaction committed */
+	if (bip->bli_flags & XFS_BLI_STALE)
+		return;
+	if (lidp->lid_flags & XFS_LID_DIRTY)
 		return;
-	}
 	xfs_trans_free_item(tp, lidp);
-	if (bip->bli_flags & XFS_BLI_HOLD) {
+	if (bip->bli_flags & XFS_BLI_HOLD)
 		bip->bli_flags &= ~XFS_BLI_HOLD;
-	}
 	XFS_BUF_SET_FSPRIVATE2(bp, NULL);
 	libxfs_putbuf(bp);
 }
@@ -339,7 +339,12 @@ libxfs_trans_binval(
 	bip = XFS_BUF_FSPRIVATE(bp, xfs_buf_log_item_t *);
 	lidp = xfs_trans_find_item(tp, (xfs_log_item_t*)bip);
 	ASSERT(lidp != NULL);
-	bip->bli_flags &= ~(XFS_BLI_DIRTY);
+	if (bip->bli_flags & XFS_BLI_STALE)
+		return;
+	XFS_BUF_UNDELAYWRITE(bp);
+	XFS_BUF_STALE(bp);
+	bip->bli_flags |= XFS_BLI_STALE;
+	bip->bli_flags &= ~XFS_BLI_DIRTY;
 	bip->bli_format.blf_flags &= ~XFS_BLI_INODE_BUF;
 	bip->bli_format.blf_flags |= XFS_BLI_CANCEL;
 	lidp->lid_flags |= XFS_LID_DIRTY;
@@ -391,11 +396,12 @@ libxfs_trans_get_buf(
 {
 	xfs_buf_t		*bp;
 	xfs_buf_log_item_t	*bip;
-	xfs_buftarg_t		bdev = { dev };
+	xfs_buftarg_t		bdev;
 
 	if (tp == NULL)
 		return libxfs_getbuf(dev, d, len);
 
+	bdev.dev = dev;
 	if (tp->t_items.lic_next == NULL)
 		bp = xfs_trans_buf_item_match(tp, &bdev, d, len);
 	else
@@ -425,6 +431,49 @@ libxfs_trans_get_buf(
 	return bp;
 }
 
+xfs_buf_t *
+libxfs_trans_getsb(
+	xfs_trans_t		*tp,
+	xfs_mount_t		*mp,
+	int			flags)
+{
+	xfs_buf_t		*bp;
+	xfs_buf_log_item_t	*bip;
+	xfs_buftarg_t		bdev;
+	int			len;
+
+	if (tp == NULL)
+		return libxfs_getsb(mp, flags);
+
+	bdev.dev = mp->m_dev;
+	len = XFS_FSS_TO_BB(mp, 1);
+	if (tp->t_items.lic_next == NULL)
+		bp = xfs_trans_buf_item_match(tp, &bdev, XFS_SB_DADDR, len);
+	else
+		bp = xfs_trans_buf_item_match_all(tp, &bdev, XFS_SB_DADDR, len);
+	if (bp != NULL) {
+		ASSERT(XFS_BUF_FSPRIVATE2(bp, xfs_trans_t *) == tp);
+		bip = XFS_BUF_FSPRIVATE(bp, xfs_buf_log_item_t *);
+		ASSERT(bip != NULL);
+		bip->bli_recur++;
+		return bp;
+	}
+
+	bp = libxfs_getsb(mp, flags);
+#ifdef XACT_DEBUG
+	fprintf(stderr, "trans_get_sb buffer %p, transaction %p\n", bp, tp);
+#endif
+
+	xfs_buf_item_init(bp, mp);
+	bip = XFS_BUF_FSPRIVATE(bp, xfs_buf_log_item_t*);
+	bip->bli_recur = 0;
+	xfs_trans_add_item(tp, (xfs_log_item_t *)bip);
+
+	/* initialize b_fsprivate2 so we can find it incore */
+	XFS_BUF_SET_FSPRIVATE2(bp, tp);
+	return bp;
+}
+
 int
 libxfs_trans_read_buf(
 	xfs_mount_t		*mp,
@@ -432,21 +481,19 @@ libxfs_trans_read_buf(
 	dev_t			dev,
 	xfs_daddr_t		blkno,
 	int			len,
-	uint			f,
+	uint			flags,
 	xfs_buf_t		**bpp)
 {
 	xfs_buf_t		*bp;
 	xfs_buf_log_item_t	*bip;
-	int			error;
-	xfs_buftarg_t		bdev = { dev };
+	xfs_buftarg_t		bdev;
 
 	if (tp == NULL) {
-		bp = libxfs_getbuf(mp->m_dev, blkno, len);
-		error = libxfs_readbufr(dev, blkno, bp, len, 0);
-		*bpp = bp;
-		return error;
+		*bpp = libxfs_readbuf(dev, blkno, len, flags);
+		return 0;
 	}
 
+	bdev.dev = dev;
 	if (tp->t_items.lic_next == NULL)
 		bp = xfs_trans_buf_item_match(tp, &bdev, blkno, len);
 	else
@@ -460,12 +507,7 @@ libxfs_trans_read_buf(
 		return 0;
 	}
 
-	bp = libxfs_getbuf(mp->m_dev, blkno, len);
-	error = libxfs_readbufr(dev, blkno, bp, len, 0);
-	if (error) {
-		*bpp = NULL;
-		return error;
-	}
+	bp = libxfs_readbuf(dev, blkno, len, flags);
 #ifdef XACT_DEBUG
 	fprintf(stderr, "trans_read_buf buffer %p, transaction %p\n", bp, tp);
 #endif
@@ -544,9 +586,7 @@ inode_item_done(
 	if (!(iip->ili_format.ilf_fields & XFS_ILOG_ALL)) {
 		ip->i_transp = NULL;	/* disassociate from transaction */
 		iip->ili_flags = 0;	/* reset all flags */
-		if (!hold)
-			goto ili_done;
-		return;
+		goto ili_done;
 	}
 
 	/*
@@ -570,21 +610,19 @@ inode_item_done(
 	ip->i_transp = NULL;	/* disassociate from transaction */
 	XFS_BUF_SET_FSPRIVATE(bp, NULL);	/* remove log item */
 	XFS_BUF_SET_FSPRIVATE2(bp, NULL);	/* remove xact ptr */
-	libxfs_writebuf_int(bp, 0);
+	libxfs_writebuf(bp, 0);
 #ifdef XACT_DEBUG
 	fprintf(stderr, "flushing dirty inode %llu, buffer %p (hold=%u)\n",
 			ip->i_ino, bp, hold);
 #endif
+ili_done:
 	if (hold) {
 		iip->ili_flags &= ~XFS_ILI_HOLD;
 		return;
-	}
-	else {
-		/*libxfs_iput(iip->ili_inode, 0);	- nathans TODO? */
-		libxfs_putbuf(bp);
+	} else {
+		libxfs_iput(ip, 0);
 	}
 
-ili_done:
 	if (ip->i_itemp)
 		kmem_zone_free(xfs_ili_zone, ip->i_itemp);
 	else
@@ -606,12 +644,13 @@ buf_item_done(
 	XFS_BUF_SET_FSPRIVATE2(bp, NULL);	/* remove xact ptr */
 
 	hold = (bip->bli_flags & XFS_BLI_HOLD);
-	if (bip->bli_flags & XFS_BLI_DIRTY) {
+	if (bip->bli_flags & (XFS_BLI_DIRTY|XFS_BLI_STALE)) {
 #ifdef XACT_DEBUG
-		fprintf(stderr, "flushing dirty buffer %p (hold=%d)\n",
+		fprintf(stderr, "flushing/staling buffer %p (hold=%d)\n",
 			bp, hold);
 #endif
-		libxfs_writebuf_int(bp, 0);
+		if (bip->bli_flags & XFS_BLI_DIRTY)
+			libxfs_writebuf_int(bp, 0);
 		if (hold)
 			bip->bli_flags &= ~XFS_BLI_HOLD;
 		else
@@ -680,6 +719,38 @@ trans_committed(
 	}
 }
 
+STATIC void
+buf_item_unlock(
+	xfs_buf_log_item_t	*bip)
+{
+	xfs_buf_t		*bp = bip->bli_buf;
+	uint			hold;
+
+	/* Clear the buffer's association with this transaction. */
+	XFS_BUF_SET_FSPRIVATE2(bip->bli_buf, NULL);
+
+	hold = bip->bli_flags & XFS_BLI_HOLD;
+	if (!hold)
+		libxfs_putbuf(bp);
+	bip->bli_flags &= ~XFS_BLI_HOLD;
+}
+
+STATIC void
+inode_item_unlock(
+	xfs_inode_log_item_t	*iip)
+{
+	xfs_inode_t		*ip = iip->ili_inode;
+	uint			hold;
+
+	/* Clear the transaction pointer in the inode. */
+	ip->i_transp = NULL;
+
+	hold = iip->ili_flags & XFS_ILI_HOLD;
+	if (!hold)
+		libxfs_iput(ip, 0);
+	iip->ili_flags = 0;
+}
+
 /*
  * Unlock each item pointed to by a descriptor in the given chunk.
  * Free descriptors pointing to items which are not dirty if freeing_chunk
@@ -712,20 +783,10 @@ xfs_trans_unlock_chunk(
 		/*
 		 * Disassociate the logged item from this transaction
 		 */
-		if (lip->li_type == XFS_LI_BUF) {
-			xfs_buf_log_item_t	*bip;
-
-			bip = (xfs_buf_log_item_t *)lidp->lid_item;
-			XFS_BUF_SET_FSPRIVATE2(bip->bli_buf, NULL);
-			bip->bli_flags &= ~XFS_BLI_HOLD;
-		}
-		else if (lip->li_type == XFS_LI_INODE) {
-			xfs_inode_log_item_t	*iip;
-
-			iip = (xfs_inode_log_item_t*)lidp->lid_item;
-			iip->ili_inode->i_transp = NULL;
-			iip->ili_flags &= ~XFS_ILI_HOLD;
-		}
+		if (lip->li_type == XFS_LI_BUF)
+			buf_item_unlock((xfs_buf_log_item_t *)lidp->lid_item);
+		else if (lip->li_type == XFS_LI_INODE)
+			inode_item_unlock((xfs_inode_log_item_t *)lidp->lid_item);
 		else {
 			fprintf(stderr, _("%s: unrecognised log item type\n"),
 				progname);
