@@ -88,14 +88,18 @@
 #define xfs_initialize_perag		libxfs_initialize_perag
 #define xfs_rtmount_init		libxfs_rtmount_init
 #define xfs_alloc_fix_freelist		libxfs_alloc_fix_freelist
-#define xfs_iread			libxfs_iread
 #define xfs_idata_realloc		libxfs_idata_realloc
 #define xfs_idestroy_fork		libxfs_idestroy_fork
+#define xfs_iread			libxfs_iread
 #define xfs_itobp			libxfs_itobp
+#define xfs_iformat			libxfs_iformat
 #define xfs_ichgtime			libxfs_ichgtime
 #define xfs_bmapi			libxfs_bmapi
+#define xfs_bmapi_single		libxfs_bmapi_single
 #define xfs_bmap_finish			libxfs_bmap_finish
+#define xfs_bmap_cancel			libxfs_bmap_cancel
 #define xfs_bmap_del_free		libxfs_bmap_del_free
+#define xfs_bmap_last_offset		libxfs_bmap_last_offset
 #define xfs_bunmapi			libxfs_bunmapi
 #define xfs_free_extent			libxfs_free_extent
 #define xfs_rtfree_extent		libxfs_rtfree_extent
@@ -112,8 +116,9 @@
 #define xfs_trans_alloc			libxfs_trans_alloc
 #define xfs_trans_commit		libxfs_trans_commit
 #define xfs_trans_cancel		libxfs_trans_cancel
-#define xfs_trans_mod_sb		libxfs_trans_mod_sb
 #define xfs_trans_reserve		libxfs_trans_reserve
+#define xfs_trans_getsb			libxfs_trans_getsb
+#define xfs_trans_mod_sb		libxfs_trans_mod_sb
 #define xfs_trans_get_buf		libxfs_trans_get_buf
 #define xfs_trans_log_buf		libxfs_trans_log_buf
 #define xfs_trans_read_buf		libxfs_trans_read_buf
@@ -153,11 +158,11 @@
 #define XFS_BUF_LOCK			0
 #define XFS_BUF_MAPPED			0
 #define XFS_BUF_TRYLOCK			0
-#define XFS_BUF_ISDONE(bp)		0
 #define XFS_BUF_GETERROR(bp)		0
-#define XFS_BUF_DONE(bp)		((void) 0)
-#define XFS_BUF_STALE(bp)		((void) 0)
-#define XFS_BUF_UNDELAYWRITE(bp)	((void) 0)
+#define XFS_BUF_DONE(bp)		((bp)->b_flags |= LIBXFS_B_UPTODATE)
+#define XFS_BUF_ISDONE(bp)		((bp)->b_flags & LIBXFS_B_UPTODATE)
+#define XFS_BUF_STALE(bp)		((bp)->b_flags |= LIBXFS_B_STALE)
+#define XFS_BUF_UNDELAYWRITE(bp)	((bp)->b_flags &= ~LIBXFS_B_DIRTY)
 #define XFS_BUF_SET_REF(a,b)		((void) 0)
 #define XFS_BUF_SET_VTYPE(a,b)		((void) 0)
 #define XFS_BUF_SET_VTYPE_REF(a,b,c)	((void) 0)
@@ -294,7 +299,6 @@ typedef struct { dev_t dev; } xfs_buftarg_t;
 #define spinlock_init(a,b)		((void) 0)
 #define spin_lock(a)			((void) 0)
 #define spin_unlock(a)			((void) 0)
-#define __return_address		__builtin_return_address(0)
 #define xfs_btree_reada_bufl(m,fsb,c)	((void) 0)
 #define xfs_btree_reada_bufs(m,fsb,c,x)	((void) 0)
 #define XFS_SB_LOCK(mp)			0
@@ -308,42 +312,40 @@ typedef struct { dev_t dev; } xfs_buftarg_t;
 #define XFS_TRANS_RESERVE_QUOTA_NBLKS(mp,tp,ip,nblks,ninos,fl)	0
 #define XFS_TRANS_UNRESERVE_QUOTA_NBLKS(mp,tp,ip,nblks,ninos,fl)	0
 
-/* These are lifted from the kernel */
-#define get_unaligned(ptr) \
-  ({ __typeof__(*(ptr)) __tmp; memmove(&__tmp, (ptr), sizeof(*(ptr))); __tmp; })
+extern void xfs_fs_repair_cmn_err(int, struct xfs_mount *, char *, ...);
+extern void xfs_fs_cmn_err(int, struct xfs_mount *, char *, ...);
 
-#define put_unaligned(val, ptr)			\
-  ({ __typeof__(*(ptr)) __tmp = (val);		\
-     memmove((ptr), &__tmp, sizeof(*(ptr)));	\
-     (void)0; })
-
-#if (__GNUC__ < 2) || ((__GNUC__ == 2) && (__GNUC_MINOR__ <= 95))
-# define xfs_fs_cmn_err(a,b,msg,args...)( fprintf(stderr, msg, ## args) )
-# define cmn_err(a,msg,args...)		( fprintf(stderr, msg, ## args) )
-# define printk(msg,args...)		( fprintf(stderr, msg, ## args) )
-#else
-# define xfs_fs_cmn_err(a,b,...)	( fprintf(stderr, __VA_ARGS__) )
-# define cmn_err(a,...)			( fprintf(stderr, __VA_ARGS__) )
-# define printk(...)			( fprintf(stderr, __VA_ARGS__) )
+#ifdef __GNUC__
+#define __return_address	__builtin_return_address(0)
 #endif
 
-#define rol32(x,y)	(((x) << (y)) | ((x) >> (32 - (y))))
-#define do_mod(a, b)	((a) % (b))
-#define do_div(n,base)	({ \
-	int __res; \
-	__res = ((unsigned long) n) % (unsigned) base; \
-	n = ((unsigned long) n) / (unsigned) base; \
-	__res; })
-
-static inline int atomicIncWithWrap(int *a, int b)
+static inline __u64 __get_unaligned64(void *ptr)
 {
-	int r = *a;
-	(*a)++;
-	if (*a == b)
-		*a = 0;
-	return r;
+	__u64 __tmp;
+	memmove(&__tmp, ptr, 8);
+	return __tmp;
 }
 
+static inline void __put_unaligned64(__u64 val, void *ptr)
+{
+	__u64 __tmp = val;
+	memmove(ptr, &__tmp, 8);
+}
+
+#define get_unaligned(ptr)	__get_unaligned64(ptr)
+#define put_unaligned(val,ptr)	__put_unaligned64(val,ptr)
+
+static inline int __do_div(unsigned long long *n, unsigned base)
+{
+	int __res;
+	__res = (int)(((unsigned long) *n) % (unsigned) base);
+	*n = ((unsigned long) *n) / (unsigned) base;
+	return __res;
+}
+#define do_div(n,base)	(__do_div((unsigned long long *)&(n), (base)))
+#define do_mod(a, b)	((a) % (b))
+#define rol32(x,y)	(((x) << (y)) | ((x) >> (32 - (y))))
+
 
 /*
  * Prototypes needed for a clean build
@@ -423,6 +425,8 @@ static inline int xfs_btree_readahead (xfs_btree_cur_t *cur, int lev, int lr)
 /* xfs_inode.c */
 int  xfs_ialloc (xfs_trans_t *, xfs_inode_t *, mode_t, nlink_t, xfs_dev_t, cred_t *,
 		xfs_prid_t, int, xfs_buf_t **, boolean_t *, xfs_inode_t **);
+int xfs_iread (xfs_mount_t *, xfs_trans_t *, xfs_ino_t, xfs_inode_t *,
+		xfs_daddr_t);
 int  xfs_iread_extents (xfs_trans_t *, xfs_inode_t *, int);
 int  xfs_imap (xfs_mount_t *, xfs_trans_t *, xfs_ino_t, xfs_imap_t *, uint);
 int  xfs_iextents_copy (xfs_inode_t *, xfs_bmbt_rec_t *, int);
