@@ -1,44 +1,28 @@
 /*
- * Copyright (c) 2000 Silicon Graphics, Inc.  All Rights Reserved.
- * 
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * Copyright (c) 2000-2001,2005 Silicon Graphics, Inc.
+ * All Rights Reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
- * 
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * 
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- * 
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- * 
- * http://www.sgi.com 
- * 
- * For further information regarding this notice, see: 
- * 
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ *
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <libxfs.h>
-#include "proto.h"
+#include <xfs/libxfs.h>
+#include <sys/stat.h>
+#include "xfs_mkfs.h"
 
 /*
  * Prototypes for internal functions.
  */
-extern long long cvtnum(int blocksize, char *s);
-extern void parseproto(xfs_mount_t *mp, xfs_inode_t *pip, char **pp,
-	char *name); 
 static long getnum(char **pp);
 static char *getstr(char **pp);
 static void fail(char *msg, int i);
@@ -46,7 +30,7 @@ static void getres(xfs_trans_t *tp, uint blocks);
 static void rsvfile(xfs_mount_t *mp, xfs_inode_t *ip, long long len);
 static int newfile(xfs_trans_t *tp, xfs_inode_t *ip, xfs_bmap_free_t *flist,
 	xfs_fsblock_t *first, int dolocal, int logit, char *buf, int len);
-static char *newregfile(char **pp, int *len); 
+static char *newregfile(char **pp, int *len);
 static void rtinit(xfs_mount_t *mp);
 static long filesize(int fd);
 
@@ -73,18 +57,18 @@ setup_proto(
 	if (!fname)
 		return dflt;
 	if ((fd = open(fname, O_RDONLY)) < 0 || (size = filesize(fd)) < 0) {
-		fprintf(stderr, "%s: failed to open %s: %s\n",
+		fprintf(stderr, _("%s: failed to open %s: %s\n"),
 			progname, fname, strerror(errno));
 		exit(1);
 	}
 	buf = malloc(size + 1);
 	if (read(fd, buf, size) < size) {
-		fprintf(stderr, "%s: read failed on %s: %s\n",
+		fprintf(stderr, _("%s: read failed on %s: %s\n"),
 			progname, fname, strerror(errno));
 		exit(1);
 	}
 	if (buf[size - 1] != '\n') {
-		fprintf(stderr, "%s: proto file %s premature EOF\n",
+		fprintf(stderr, _("%s: proto file %s premature EOF\n"),
 			progname, fname);
 		exit(1);
 	}
@@ -121,7 +105,7 @@ void
 res_failed(
 	int	i)
 {
-	fail("cannot reserve space", i);
+	fail(_("cannot reserve space"), i);
 }
 
 static void
@@ -147,7 +131,7 @@ static char *
 getstr(
 	char	**pp)
 {
-	int	c;
+	char	c;
 	char	*p;
 	char	*rval;
 
@@ -173,8 +157,8 @@ getstr(
 			return rval;
 		}
 	}
-	if (!c) {
-		fprintf(stderr, "%s: premature EOF in prototype file\n",
+	if (c != '\0') {
+		fprintf(stderr, _("%s: premature EOF in prototype file\n"),
 			progname);
 		exit(1);
 	}
@@ -193,7 +177,7 @@ rsvfile(
 	error = libxfs_alloc_file_space(ip, 0, llen, 1, 0);
 
 	if (error) {
-		fail("error reserving space for a file", error);
+		fail(_("error reserving space for a file"), error);
 		exit(1);
 	}
 
@@ -205,17 +189,17 @@ rsvfile(
 	libxfs_trans_ijoin(tp, ip, 0);
 	libxfs_trans_ihold(tp, ip);
 
-	ip->i_d.di_mode &= ~ISUID;
+	ip->i_d.di_mode &= ~S_ISUID;
 
 	/*
 	 * Note that we don't have to worry about mandatory
 	 * file locking being disabled here because we only
-	 * clear the ISGID bit if the Group execute bit is
+	 * clear the S_ISGID bit if the Group execute bit is
 	 * on, but if it was on then mandatory locking wouldn't
 	 * have been enabled.
 	 */
-	if (ip->i_d.di_mode & (IEXEC >> 3))
-		ip->i_d.di_mode &= ~ISGID;
+	if (ip->i_d.di_mode & S_IXGRP)
+		ip->i_d.di_mode &= ~S_ISGID;
 
 	libxfs_ichgtime(ip, XFS_ICHGTIME_MOD | XFS_ICHGTIME_CHG);
 
@@ -262,10 +246,11 @@ newfile(
 		error = libxfs_bmapi(tp, ip, 0, nb, XFS_BMAPI_WRITE, first, nb,
 				&map, &nmap, flist);
 		if (error) {
-			fail("error allocating space for a file", error);
+			fail(_("error allocating space for a file"), error);
 		}
 		if (nmap != 1) {
-			fprintf(stderr, "%s: cannot allocate space for file\n",
+			fprintf(stderr,
+				_("%s: cannot allocate space for file\n"),
 				progname);
 			exit(1);
 		}
@@ -278,7 +263,7 @@ newfile(
 		if (logit)
 			libxfs_trans_log_buf(tp, bp, 0, XFS_BUF_COUNT(bp) - 1);
 		else
-			libxfs_writebuf(bp, 1);
+			libxfs_writebuf(bp, LIBXFS_EXIT_ON_FAILURE);
 	}
 	ip->i_d.di_size = len;
 	return flags;
@@ -296,14 +281,14 @@ newregfile(
 
 	fname = getstr(pp);
 	if ((fd = open(fname, O_RDONLY)) < 0 || (size = filesize(fd)) < 0) {
-		fprintf(stderr, "%s: cannot open %s: %s\n",
+		fprintf(stderr, _("%s: cannot open %s: %s\n"),
 			progname, fname, strerror(errno));
 		exit(1);
 	}
 	if ((*len = (int)size)) {
 		buf = malloc(size);
 		if (read(fd, buf, size) < size) {
-			fprintf(stderr, "%s: read failed on %s: %s\n",
+			fprintf(stderr, _("%s: read failed on %s: %s\n"),
 				progname, fname, strerror(errno));
 			exit(1);
 		}
@@ -334,7 +319,7 @@ newdirent(
 		error = libxfs_dir_createname(tp, pip, name, namelen,
 						inum, first, flist, total);
 	if (error)
-		fail("directory createname error", error);
+		fail(_("directory createname error"), error);
 }
 
 static void
@@ -351,13 +336,14 @@ newdirectory(
 	else
 		error = libxfs_dir_init(tp, dp, pdp);
 	if (error)
-		fail("directory create error", error);
+		fail(_("directory create error"), error);
 }
 
-void
+static void
 parseproto(
 	xfs_mount_t	*mp,
 	xfs_inode_t	*pip,
+	struct fsxattr	*fsxp,
 	char		**pp,
 	char		*name)
 {
@@ -415,7 +401,8 @@ parseproto(
 		fmt = IF_FIFO;
 		break;
 	default:
-		fprintf(stderr, "%s: bad format string %s\n", progname, mstr);
+		fprintf(stderr, _("%s: bad format string %s\n"),
+			progname, mstr);
 		exit(1);
 	}
 	mode = 0;
@@ -423,26 +410,28 @@ parseproto(
 	case '-':
 		break;
 	case 'u':
-		mode |= ISUID;
+		mode |= S_ISUID;
 		break;
 	default:
-		fprintf(stderr, "%s: bad format string %s\n", progname, mstr);
+		fprintf(stderr, _("%s: bad format string %s\n"),
+			progname, mstr);
 		exit(1);
 	}
 	switch (mstr[2]) {
 	case '-':
 		break;
 	case 'g':
-		mode |= ISGID;
+		mode |= S_ISGID;
 		break;
 	default:
-		fprintf(stderr, "%s: bad format string %s\n", progname, mstr);
+		fprintf(stderr, _("%s: bad format string %s\n"),
+			progname, mstr);
 		exit(1);
 	}
 	val = 0;
 	for (i = 3; i < 6; i++) {
 		if (mstr[i] < '0' || mstr[i] > '7') {
-			fprintf(stderr, "%s: bad format string %s\n",
+			fprintf(stderr, _("%s: bad format string %s\n"),
 				progname, mstr);
 			exit(1);
 		}
@@ -458,10 +447,10 @@ parseproto(
 	case IF_REGULAR:
 		buf = newregfile(pp, &len);
 		getres(tp, XFS_B_TO_FSB(mp, len));
-		error = libxfs_inode_alloc(&tp, pip, mode|IFREG, 1,
-					mp->m_dev, &creds, &ip);
+		error = libxfs_inode_alloc(&tp, pip, mode|S_IFREG, 1, 0,
+					   &creds, fsxp, &ip);
 		if (error)
-			fail("Inode allocation failed", error);
+			fail(_("Inode allocation failed"), error);
 		flags |= newfile(tp, ip, &flist, &first, 0, 0, buf, len);
 		if (buf)
 			free(buf);
@@ -473,13 +462,13 @@ parseproto(
 
 	case IF_RESERVED:			/* pre-allocated space only */
 		value = getstr(pp);
-		llen = cvtnum(mp->m_sb.sb_blocksize, value);
+		llen = cvtnum(mp->m_sb.sb_blocksize, mp->m_sb.sb_sectsize, value);
 		getres(tp, XFS_B_TO_FSB(mp, llen));
 
-		error = libxfs_inode_alloc(&tp, pip, mode|IFREG, 1,
-						mp->m_dev, &creds, &ip);
+		error = libxfs_inode_alloc(&tp, pip, mode|S_IFREG, 1, 0,
+					  &creds, fsxp, &ip);
 		if (error)
-			fail("Inode pre-allocation failed", error);
+			fail(_("Inode pre-allocation failed"), error);
 
 		libxfs_trans_ijoin(tp, pip, 0);
 
@@ -490,7 +479,7 @@ parseproto(
 
 		error = libxfs_bmap_finish(&tp, &flist, first, &committed);
 		if (error)
-			fail("Pre-allocated file creation failed", error);
+			fail(_("Pre-allocated file creation failed"), error);
 		libxfs_trans_commit(tp, 0, NULL);
 		rsvfile(mp, ip, llen);
 		return;
@@ -499,10 +488,10 @@ parseproto(
 		getres(tp, 0);
 		majdev = (int)getnum(pp);
 		mindev = (int)getnum(pp);
-		error = libxfs_inode_alloc(&tp, pip, mode|IFBLK, 1,
-				makedev(majdev, mindev), &creds, &ip);
+		error = libxfs_inode_alloc(&tp, pip, mode|S_IFBLK, 1,
+				IRIX_MKDEV(majdev, mindev), &creds, fsxp, &ip);
 		if (error) {
-			fail("Inode allocation failed", error);
+			fail(_("Inode allocation failed"), error);
 		}
 		libxfs_trans_ijoin(tp, pip, 0);
 		i = strlen(name);
@@ -515,10 +504,10 @@ parseproto(
 		getres(tp, 0);
 		majdev = (int)getnum(pp);
 		mindev = (int)getnum(pp);
-		error = libxfs_inode_alloc(&tp, pip, mode|IFCHR, 1,
-				makedev(majdev, mindev), &creds, &ip);
+		error = libxfs_inode_alloc(&tp, pip, mode|S_IFCHR, 1,
+				IRIX_MKDEV(majdev, mindev), &creds, fsxp, &ip);
 		if (error)
-			fail("Inode allocation failed", error);
+			fail(_("Inode allocation failed"), error);
 		libxfs_trans_ijoin(tp, pip, 0);
 		i = strlen(name);
 		newdirent(mp, tp, pip, name, i, ip->i_ino, &first, &flist, 1);
@@ -528,10 +517,10 @@ parseproto(
 
 	case IF_FIFO:
 		getres(tp, 0);
-		error = libxfs_inode_alloc(&tp, pip, mode|IFIFO, 1,
-				mp->m_dev, &creds, &ip);
+		error = libxfs_inode_alloc(&tp, pip, mode|S_IFIFO, 1, 0,
+				&creds, fsxp, &ip);
 		if (error)
-			fail("Inode allocation failed", error);
+			fail(_("Inode allocation failed"), error);
 		libxfs_trans_ijoin(tp, pip, 0);
 		i = strlen(name);
 		newdirent(mp, tp, pip, name, i, ip->i_ino, &first, &flist, 1);
@@ -541,10 +530,10 @@ parseproto(
 		buf = getstr(pp);
 		len = (int)strlen(buf);
 		getres(tp, XFS_B_TO_FSB(mp, len));
-		error = libxfs_inode_alloc(&tp, pip, mode|IFLNK, 1,
-				mp->m_dev, &creds, &ip);
+		error = libxfs_inode_alloc(&tp, pip, mode|S_IFLNK, 1, 0,
+				&creds, fsxp, &ip);
 		if (error)
-			fail("Inode allocation failed", error);
+			fail(_("Inode allocation failed"), error);
 		flags |= newfile(tp, ip, &flist, &first, 1, 1, buf, len);
 		libxfs_trans_ijoin(tp, pip, 0);
 		i = strlen(name);
@@ -553,10 +542,10 @@ parseproto(
 		break;
 	case IF_DIRECTORY:
 		getres(tp, 0);
-		error = libxfs_inode_alloc(&tp, pip, mode|IFDIR, 1,
-				mp->m_dev, &creds, &ip);
+		error = libxfs_inode_alloc(&tp, pip, mode|S_IFDIR, 1, 0,
+				&creds, fsxp, &ip);
 		if (error)
-			fail("Inode allocation failed", error);
+			fail(_("Inode allocation failed"), error);
 		ip->i_d.di_nlink++;		/* account for . */
 		if (!pip) {
 			pip = ip;
@@ -577,7 +566,7 @@ parseproto(
 		libxfs_trans_log_inode(tp, ip, flags);
 		error = libxfs_bmap_finish(&tp, &flist, first, &committed);
 		if (error)
-			fail("Directory creation failed", error);
+			fail(_("Directory creation failed"), error);
 		libxfs_trans_ihold(tp, ip);
 		libxfs_trans_commit(tp, 0, NULL);
 		/*
@@ -589,9 +578,11 @@ parseproto(
 		tp = NULL;
 		for (;;) {
 			name = getstr(pp);
+			if (!name)
+				break;
 			if (strcmp(name, "$") == 0)
 				break;
-			parseproto(mp, ip, pp, name);
+			parseproto(mp, ip, fsxp, pp, name);
 		}
 		libxfs_iput(ip, 0);
 		return;
@@ -599,11 +590,21 @@ parseproto(
 	libxfs_trans_log_inode(tp, ip, flags);
 	error = libxfs_bmap_finish(&tp, &flist, first, &committed);
 	if (error) {
-		fail("Error encountered creating file from prototype", error);
+		fail(_("Error encountered creating file from prototype file"),
+			error);
 	}
 	libxfs_trans_commit(tp, 0, NULL);
 }
 
+void
+parse_proto(
+	xfs_mount_t	*mp,
+	struct fsxattr	*fsx,
+	char		**pp)
+{
+	parseproto(mp, NULL, fsx, pp, NULL);
+}
+
 /*
  * Allocate the realtime bitmap and summary inodes, and fill in data if any.
  */
@@ -625,7 +626,8 @@ rtinit(
 	xfs_inode_t	*rbmip;
 	xfs_inode_t	*rsumip;
 	xfs_trans_t	*tp;
-	cred_t		creds;
+	struct cred	creds;
+	struct fsxattr	fsxattrs;
 
 	/*
 	 * First, allocate the inodes.
@@ -634,10 +636,11 @@ rtinit(
 	if ((i = libxfs_trans_reserve(tp, MKFS_BLOCKRES_INODE, 0, 0, 0, 0)))
 		res_failed(i);
 	bzero(&creds, sizeof(creds));
-	error = libxfs_inode_alloc(&tp, mp->m_rootip, IFREG, 1,
-				mp->m_dev, &creds, &rbmip);
+	bzero(&fsxattrs, sizeof(fsxattrs));
+	error = libxfs_inode_alloc(&tp, NULL, S_IFREG, 1, 0,
+					&creds, &fsxattrs, &rbmip);
 	if (error) {
-		fail("Realtime bitmap inode allocation failed", error);
+		fail(_("Realtime bitmap inode allocation failed"), error);
 	}
 	/*
 	 * Do our thing with rbmip before allocating rsumip,
@@ -652,10 +655,10 @@ rtinit(
 	libxfs_mod_sb(tp, XFS_SB_RBMINO);
 	libxfs_trans_ihold(tp, rbmip);
 	mp->m_rbmip = rbmip;
-	error = libxfs_inode_alloc(&tp, mp->m_rootip, IFREG, 1,
-				mp->m_dev, &creds, &rsumip);
+	error = libxfs_inode_alloc(&tp, NULL, S_IFREG, 1, 0,
+					&creds, &fsxattrs, &rsumip);
 	if (error) {
-		fail("Realtime bitmap inode allocation failed", error);
+		fail(_("Realtime summary inode allocation failed"), error);
 	}
 	mp->m_sb.sb_rsumino = rsumip->i_ino;
 	rsumip->i_d.di_size = mp->m_rsumsize;
@@ -672,6 +675,7 @@ rtinit(
 			(XFS_BM_MAXLEVELS(mp, XFS_DATA_FORK) - 1), 0, 0, 0, 0)))
 		res_failed(i);
 	libxfs_trans_ijoin(tp, rbmip, 0);
+	libxfs_trans_ihold(tp, rbmip);
 	bno = 0;
 	XFS_BMAP_INIT(&flist, &first);
 	while (bno < mp->m_sb.sb_rbmblocks) {
@@ -681,7 +685,8 @@ rtinit(
 				XFS_BMAPI_WRITE, &first, mp->m_sb.sb_rbmblocks,
 				map, &nmap, &flist);
 		if (error) {
-			fail("Allocation of the realtime bitmap failed", error);
+			fail(_("Allocation of the realtime bitmap failed"),
+				error);
 		}
 		for (i = 0, ep = map; i < nmap; i++, ep++) {
 			libxfs_device_zero(mp->m_dev,
@@ -693,9 +698,10 @@ rtinit(
 
 	error = libxfs_bmap_finish(&tp, &flist, first, &committed);
 	if (error) {
-		fail("Allocation of the realtime bitmap failed", error);
+		fail(_("Completion of the realtime bitmap failed"), error);
 	}
 	libxfs_trans_commit(tp, 0, NULL);
+
 	/*
 	 * Give the summary file some zero-filled blocks.
 	 */
@@ -706,6 +712,7 @@ rtinit(
 			0, 0, 0, 0)))
 		res_failed(i);
 	libxfs_trans_ijoin(tp, rsumip, 0);
+	libxfs_trans_ihold(tp, rsumip);
 	bno = 0;
 	XFS_BMAP_INIT(&flist, &first);
 	while (bno < nsumblocks) {
@@ -715,7 +722,8 @@ rtinit(
 				XFS_BMAPI_WRITE, &first, nsumblocks,
 				map, &nmap, &flist);
 		if (error) {
-			fail("Allocation of the realtime bitmap failed", error);
+			fail(_("Allocation of the realtime summary failed"),
+				error);
 		}
 		for (i = 0, ep = map; i < nmap; i++, ep++) {
 			libxfs_device_zero(mp->m_dev,
@@ -726,9 +734,10 @@ rtinit(
 	}
 	error = libxfs_bmap_finish(&tp, &flist, first, &committed);
 	if (error) {
-		fail("Allocation of the realtime bitmap failed", error);
+		fail(_("Completion of the realtime summary failed"), error);
 	}
 	libxfs_trans_commit(tp, 0, NULL);
+
 	/*
 	 * Free the whole area using transactions.
 	 * Do one transaction per bitmap block.
@@ -742,11 +751,12 @@ rtinit(
 			bno + NBBY * mp->m_sb.sb_blocksize);
 		error = libxfs_rtfree_extent(tp, bno, (xfs_extlen_t)(ebno-bno));
 		if (error) {
-			fail("Error initializing the realtime bitmap", error);
+			fail(_("Error initializing the realtime space"),
+				error);
 		}
 		error = libxfs_bmap_finish(&tp, &flist, first, &committed);
 		if (error) {
-			fail("Error initializing the realtime bitmap", error);
+			fail(_("Error completing the realtime space"), error);
 		}
 		libxfs_trans_commit(tp, 0, NULL);
 	}
