@@ -1,33 +1,19 @@
 /*
- * Copyright (c) 2000-2002 Silicon Graphics, Inc.  All Rights Reserved.
+ * Copyright (c) 2000-2002,2005 Silicon Graphics, Inc.
+ * All Rights Reserved.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- *
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- *
- * http://www.sgi.com
- *
- * For further information regarding this notice, see:
- *
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /*
@@ -181,6 +167,16 @@ typedef struct rt_extent_tree_node  {
 #define XR_E_FS_MAP	7	/* extent used by fs space/inode maps */
 #define XR_E_BAD_STATE	8
 
+/* extent states, in 64 bit word chunks */
+#define	XR_E_UNKNOWN_LL		0x0000000000000000LL
+#define	XR_E_FREE1_LL		0x1111111111111111LL
+#define	XR_E_FREE_LL		0x2222222222222222LL
+#define	XR_E_INUSE_LL		0x3333333333333333LL
+#define	XR_E_INUSE_FS_LL	0x4444444444444444LL
+#define	XR_E_MULT_LL		0x5555555555555555LL
+#define	XR_E_INO_LL		0x6666666666666666LL
+#define	XR_E_FS_MAP_LL		0x7777777777777777LL
+
 /* separate state bit, OR'ed into high (4th) bit of ex_state field */
 
 #define XR_E_WRITTEN	0x8	/* extent has been written out, can't reclaim */
@@ -241,9 +237,18 @@ void		add_dup_extent(xfs_agnumber_t agno,
 				xfs_agblock_t startblock,
 				xfs_extlen_t blockcount);
 
-int		search_dup_extent(xfs_mount_t *mp,
-				xfs_agnumber_t agno,
-				xfs_agblock_t agbno);
+extern avltree_desc_t   **extent_tree_ptrs;
+/* ARGSUSED */
+static inline int
+search_dup_extent(xfs_mount_t *mp, xfs_agnumber_t agno, xfs_agblock_t agbno)
+{
+	ASSERT(agno < glob_agcount);
+
+	if (avl_findrange(extent_tree_ptrs[agno], agbno) != NULL)
+		return(1);
+
+	return(0);
+}
 
 void		add_rt_dup_extent(xfs_drtbno_t	startblock,
 				xfs_extlen_t	blockcount);
@@ -362,8 +367,18 @@ void		free_inode_rec(xfs_agnumber_t agno, ino_tree_node_t *ino_rec);
  */
 void		get_inode_rec(xfs_agnumber_t agno, ino_tree_node_t *ino_rec);
 
-ino_tree_node_t *findfirst_inode_rec(xfs_agnumber_t agno);
-ino_tree_node_t *find_inode_rec(xfs_agnumber_t agno, xfs_agino_t ino);
+extern avltree_desc_t     **inode_tree_ptrs;
+static inline ino_tree_node_t *
+findfirst_inode_rec(xfs_agnumber_t agno)
+{
+	return((ino_tree_node_t *) inode_tree_ptrs[agno]->avl_firstino);
+}
+static inline ino_tree_node_t *
+find_inode_rec(xfs_agnumber_t agno, xfs_agino_t ino)
+{
+	return((ino_tree_node_t *)
+		avl_findrange(inode_tree_ptrs[agno], ino));
+}
 void		find_inode_rec_range(xfs_agnumber_t agno,
 			xfs_agino_t start_ino, xfs_agino_t end_ino,
 			ino_tree_node_t **first, ino_tree_node_t **last);
@@ -478,17 +493,17 @@ void			clear_uncertain_ino_cache(xfs_agnumber_t agno);
  */
 #define set_inode_free(ino_rec, ino_offset) \
 	XFS_INOCF_SET_CF((ino_rec), (ino_offset)), \
-	XFS_INOBT_SET_FREE((ino_rec), (ino_offset),(ARCH_NOCONVERT))
+	XFS_INOBT_SET_FREE((ino_rec), (ino_offset))
 
 #define set_inode_used(ino_rec, ino_offset) \
 	XFS_INOCF_SET_CF((ino_rec), (ino_offset)), \
-	XFS_INOBT_CLR_FREE((ino_rec), (ino_offset),(ARCH_NOCONVERT))
+	XFS_INOBT_CLR_FREE((ino_rec), (ino_offset))
 
 #define is_inode_used(ino_rec, ino_offset)	\
-	!XFS_INOBT_IS_FREE((ino_rec), (ino_offset),(ARCH_NOCONVERT))
+	!XFS_INOBT_IS_FREE((ino_rec), (ino_offset))
 
 #define is_inode_free(ino_rec, ino_offset)	\
-	XFS_INOBT_IS_FREE((ino_rec), (ino_offset),(ARCH_NOCONVERT))
+	XFS_INOBT_IS_FREE((ino_rec), (ino_offset))
 
 /*
  * add_inode_reached() is set on inode I only if I has been reached
@@ -500,12 +515,55 @@ void			clear_uncertain_ino_cache(xfs_agnumber_t agno);
  * an inode that we've counted is removed.
  */
 
-void		add_inode_reached(ino_tree_node_t *ino_rec, int ino_offset);
-void		add_inode_ref(ino_tree_node_t *ino_rec, int ino_offset);
-void		drop_inode_ref(ino_tree_node_t *ino_rec, int ino_offset);
-int		is_inode_reached(ino_tree_node_t *ino_rec, int ino_offset);
-int		is_inode_referenced(ino_tree_node_t *ino_rec, int ino_offset);
-__uint32_t	num_inode_references(ino_tree_node_t *ino_rec, int ino_offset);
+static inline int
+is_inode_reached(ino_tree_node_t *ino_rec, int ino_offset)
+{
+	ASSERT(ino_rec->ino_un.backptrs != NULL);
+	return(XFS_INO_RCHD_IS_RCHD(ino_rec, ino_offset));
+}
+
+static inline void
+add_inode_reached(ino_tree_node_t *ino_rec, int ino_offset)
+{
+	ASSERT(ino_rec->ino_un.backptrs != NULL);
+
+	ino_rec->ino_un.backptrs->nlinks[ino_offset]++;
+	XFS_INO_RCHD_SET_RCHD(ino_rec, ino_offset);
+
+	ASSERT(is_inode_reached(ino_rec, ino_offset));
+}
+
+static inline void
+add_inode_ref(ino_tree_node_t *ino_rec, int ino_offset)
+{
+	ASSERT(ino_rec->ino_un.backptrs != NULL);
+
+	ino_rec->ino_un.backptrs->nlinks[ino_offset]++;
+}
+
+static inline void
+drop_inode_ref(ino_tree_node_t *ino_rec, int ino_offset)
+{
+	ASSERT(ino_rec->ino_un.backptrs != NULL);
+	ASSERT(ino_rec->ino_un.backptrs->nlinks[ino_offset] > 0);
+
+	if (--ino_rec->ino_un.backptrs->nlinks[ino_offset] == 0)
+		XFS_INO_RCHD_CLR_RCHD(ino_rec, ino_offset);
+}
+
+static inline int
+is_inode_referenced(ino_tree_node_t *ino_rec, int ino_offset)
+{
+	ASSERT(ino_rec->ino_un.backptrs != NULL);
+	return(ino_rec->ino_un.backptrs->nlinks[ino_offset] > 0);
+}
+
+static inline __uint32_t
+num_inode_references(ino_tree_node_t *ino_rec, int ino_offset)
+{
+	ASSERT(ino_rec->ino_un.backptrs != NULL);
+	return(ino_rec->ino_un.backptrs->nlinks[ino_offset]);
+}
 
 /*
  * has an inode been processed for phase 6 (reference count checking)?
