@@ -1,33 +1,19 @@
 /*
- * Copyright (c) 2000 Silicon Graphics, Inc.  All Rights Reserved.
- * 
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * Copyright (c) 2000-2001,2005 Silicon Graphics, Inc.
+ * All Rights Reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
- * 
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * 
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- * 
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- * 
- * http://www.sgi.com 
- * 
- * For further information regarding this notice, see: 
- * 
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ *
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <libxfs.h>
@@ -80,8 +66,8 @@ walk_unlinked_list(xfs_mount_t *mp, xfs_agnumber_t agno, xfs_agino_t start_ino)
 						XR_E_INO);
 					break;
 				case XR_E_BAD_STATE:
-					do_error(
-						"bad state in block map %d\n",
+					do_error(_(
+						"bad state in block map %d\n"),
 						state);
 					abort();
 					break;
@@ -120,13 +106,12 @@ process_agi_unlinked(xfs_mount_t *mp, xfs_agnumber_t agno)
 	int err = 0;
 	int agi_dirty = 0;
 
-	bp = libxfs_readbuf(mp->m_dev, XFS_AG_DADDR(mp, agno, XFS_AGI_DADDR),
-				mp->m_sb.sb_sectsize/BBSIZE, 0);
-	if (!bp) {
-		do_error("cannot read agi block %lld for ag %u\n",
-			XFS_AG_DADDR(mp, agno, XFS_AGI_DADDR), agno);
-		exit(1);
-	}
+	bp = libxfs_readbuf(mp->m_dev,
+			XFS_AG_DADDR(mp, agno, XFS_AGI_DADDR(mp)),
+			mp->m_sb.sb_sectsize/BBSIZE, 0);
+	if (!bp)
+		do_error(_("cannot read agi block %lld for ag %u\n"),
+			XFS_AG_DADDR(mp, agno, XFS_AGI_DADDR(mp)), agno);
 
 	agip = XFS_BUF_TO_AGI(bp);
 
@@ -134,20 +119,21 @@ process_agi_unlinked(xfs_mount_t *mp, xfs_agnumber_t agno)
 
 	for (i = 0; i < XFS_AGI_UNLINKED_BUCKETS; i++)  {
 		if (INT_GET(agip->agi_unlinked[i], ARCH_CONVERT) != NULLAGINO)  {
-			err += walk_unlinked_list(mp, agno,
-						INT_GET(agip->agi_unlinked[i], ARCH_CONVERT));
+			err += walk_unlinked_list(mp, agno, INT_GET(
+					agip->agi_unlinked[i], ARCH_CONVERT));
 			/*
 			 * clear the list
 			 */
 			if (!no_modify)  {
-				INT_SET(agip->agi_unlinked[i], ARCH_CONVERT, NULLAGINO);
+				INT_SET(agip->agi_unlinked[i], ARCH_CONVERT,
+					NULLAGINO);
 				agi_dirty = 1;
 			}
 		}
 	}
 
 	if (err)
-		do_warn("error following ag %d unlinked list\n", agno);
+		do_warn(_("error following ag %d unlinked list\n"), agno);
 
 	ASSERT(agi_dirty == 0 || (agi_dirty && !no_modify));
 
@@ -162,11 +148,11 @@ phase3(xfs_mount_t *mp)
 {
 	int i, j;
 
-	printf("Phase 3 - for each AG...\n");
+	do_log(_("Phase 3 - for each AG...\n"));
 	if (!no_modify)
-		printf("        - scan and clear agi unlinked lists...\n");
+		do_log(_("        - scan and clear agi unlinked lists...\n"));
 	else
-		printf("        - scan (but don't clear) agi unlinked lists...\n");
+		do_log(_("        - scan (but don't clear) agi unlinked lists...\n"));
 
 	/*
 	 * first, let's look at the possibly bogus inodes
@@ -181,13 +167,13 @@ phase3(xfs_mount_t *mp)
 
 	/* ok, now that the tree's ok, let's take a good look */
 
-	printf(
-	    "        - process known inodes and perform inode discovery...\n");
+	do_log(_(
+	    "        - process known inodes and perform inode discovery...\n"));
 
 	for (i = 0; i < mp->m_sb.sb_agcount; i++)  {
-		do_log("        - agno = %d\n", i);
+		do_log(_("        - agno = %d\n"), i);
 		/*
-		 * turn on directory processing (inode discovery) and 
+		 * turn on directory processing (inode discovery) and
 		 * attribute processing (extra_attr_check)
 		 */
 		process_aginodes(mp, i, 1, 0, 1);
@@ -196,7 +182,7 @@ phase3(xfs_mount_t *mp)
 	/*
 	 * process newly discovered inode chunks
 	 */
-	printf("        - process newly discovered inodes...\n");
+	do_log(_("        - process newly discovered inodes...\n"));
 	do  {
 		/*
 		 * have to loop until no ag has any uncertain
@@ -212,4 +198,3 @@ phase3(xfs_mount_t *mp)
 		}
 	} while (j != 0);
 }
-
