@@ -25,6 +25,7 @@
 #include "protos.h"
 #include "incore.h"
 #include "err_protos.h"
+#include "prefetch.h"
 
 #define	rounddown(x, y)	(((x)/(y))*(y))
 
@@ -52,6 +53,16 @@ char *o_opts[] = {
 	"assume_xfs",
 #define PRE_65_BETA	1
 	"fs_is_pre_65_beta",
+#define	IHASH_SIZE	2
+	"ihash",
+#define	BHASH_SIZE	3
+	"bhash",
+#define	PREFETCH_INO_CNT	4
+	"pfino",
+#define	PREFETCH_DIR_CNT	5
+	"pfdir",
+#define	PREFETCH_AIO_CNT	6
+	"pfaio",
 	NULL
 };
 
@@ -176,7 +187,7 @@ process_args(int argc, char **argv)
 	 * XXX have to add suboption processing here
 	 * attributes, quotas, nlinks, aligned_inos, sb_fbits
 	 */
-	while ((c = getopt(argc, argv, "o:fl:r:LnDvVd")) != EOF)  {
+	while ((c = getopt(argc, argv, "o:fl:r:LnDvVdP")) != EOF)  {
 		switch (c) {
 		case 'D':
 			dumpcore = 1;
@@ -202,6 +213,21 @@ process_args(int argc, char **argv)
 							PRE_65_BETA);
 					pre_65_beta = 1;
 					break;
+				case IHASH_SIZE:
+					libxfs_ihash_size = (int) strtol(val, 0, 0);
+					break;
+				case BHASH_SIZE:
+					libxfs_bhash_size = (int) strtol(val, 0, 0);
+					break;
+				case PREFETCH_INO_CNT:
+					libxfs_lio_ino_count = (int) strtol(val, 0, 0);
+					break;
+				case PREFETCH_DIR_CNT:
+					libxfs_lio_dir_count = (int) strtol(val, 0, 0);
+					break;
+				case PREFETCH_AIO_CNT:
+					libxfs_lio_aio_count = (int) strtol(val, 0, 0);
+					break;
 				default:
 					unknown('o', val);
 					break;
@@ -234,6 +260,9 @@ process_args(int argc, char **argv)
 		case 'V':
 			printf(_("%s version %s\n"), progname, VERSION);
 			exit(0);
+		case 'P':
+			do_prefetch ^= 1;
+			break;
 		case '?':
 			usage();
 		}
@@ -244,6 +273,11 @@ process_args(int argc, char **argv)
 
 	if ((fs_name = argv[optind]) == NULL)
 		usage();
+
+	if (!isa_file) {
+		if ((fs_name = libxfs_findrawpath(fs_name)) == NULL)
+			do_error("couldn't find raw device for %s\n", fs_name);
+	}
 }
 
 void
@@ -496,9 +530,19 @@ main(int argc, char **argv)
 
 	phase2(mp);
 
+	if (verbose)
+		libxfs_report(stderr);
 	phase3(mp);
+	if (verbose)
+		libxfs_report(stderr);
 
 	phase4(mp);
+	if (verbose)
+		libxfs_report(stderr);
+
+	/* XXX: nathans - something in phase4 ain't playing by */
+	/* the buffer cache rules.. why doesn't IRIX hit this? */
+	libxfs_bcache_purge();
 
 	if (no_modify)
 		printf(_("No modify flag set, skipping phase 5\n"));
@@ -507,6 +551,8 @@ main(int argc, char **argv)
 
 	if (!bad_ino_btree)  {
 		phase6(mp);
+		if (verbose)
+			libxfs_report(stderr);
 
 		phase7(mp);
 	} else  {
@@ -568,6 +614,9 @@ _("Warning:  project quota information would be cleared.\n"
 		}
 	}
 
+	if (verbose)
+		libxfs_report(stderr);
+
 	if (no_modify)  {
 		do_log(
 	_("No modify flag set, skipping filesystem flush and exiting.\n"));
@@ -577,6 +626,12 @@ _("Warning:  project quota information would be cleared.\n"
 		return(0);
 	}
 
+	/*
+	 * Done, flush all cached buffers and inodes.
+	 */
+	libxfs_icache_purge();
+	libxfs_bcache_purge();
+
 	/*
 	 * Clear the quota flags if they're on.
 	 */
